/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
'use strict';
define(["require", "exports", 'vs/base/node/mono', 'fs', 'child_process', 'monaco'], function (require, exports, monoHelper, fs, cp, monaco) {
    var omnisharpEnv = 'OMNISHARP';
    var isWindows = /^win/.test(process.platform);
    function launch(configService, cwd, args) {
        return new Promise(function (resolve, reject) {
            try {
                (isWindows ? launchWindows(configService, cwd, args) : launchNix(configService, cwd, args)).then(function (value) {
                    // async error - when target not not ENEOT
                    value.process.on('error', reject);
                    // success after a short freeing event loop
                    setTimeout(function () {
                        resolve(value);
                    }, 0);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    exports.default = launch;
    function launchWindows(configService, cwd, args) {
        return getOmnisharpPath(configService).then(function (command) {
            args = args.slice(0);
            args.unshift(command);
            args = [[
                    '/s',
                    '/c',
                    '"' + args.map(function (arg) { return /^[^"].* .*[^"]/.test(arg) ? "\"" + arg + "\"" : arg; }).join(' ') + '"'
                ].join(' ')];
            var process = cp.spawn('cmd', args, {
                windowsVerbatimArguments: true,
                detached: false,
                // env: details.env,
                cwd: cwd
            });
            return {
                process: process,
                command: command
            };
        });
    }
    function launchNix(configService, cwd, args) {
        return new Promise(function (resolve, reject) {
            monoHelper.hasMono('>=4.0.1').then(function (hasIt) {
                if (!hasIt) {
                    reject(new Error('Cannot start Omnisharp because Mono version >=4.0.1 is required. See http://go.microsoft.com/fwlink/?linkID=534832#_20001'));
                }
                else {
                    resolve();
                }
            });
        }).then(function (_) {
            return getOmnisharpPath(configService);
        }).then(function (command) {
            var process = cp.spawn(command, args, {
                detached: false,
                // env: details.env,
                cwd: cwd
            });
            return {
                process: process,
                command: command
            };
        });
    }
    function getOmnisharpPath(configService) {
        return new Promise(function (resolve, reject) {
            configService.loadConfiguration('csharp').then(function (config) {
                // from configuration
                if (config && typeof config['omnisharp'] === 'string') {
                    return config['omnisharp'];
                }
            }, function (err) { ; }).then(function (value) {
                if (value) {
                    return value;
                }
                // form enviroment variable
                if (typeof process.env[omnisharpEnv] === 'string') {
                    console.warn('[deprecated] use workspace or user settings with "csharp.omnisharp":"/path/to/omnisharp"');
                    return process.env[omnisharpEnv];
                }
            }).then(function (value) {
                if (value) {
                    return value;
                }
                // bundled version of Omnisharp
                return new Promise(function (resolve, reject) {
                    var local = monaco.URI.parse(require.toUrl('../bin/omnisharp')).fsPath;
                    local = isWindows
                        ? local + '.cmd'
                        : local;
                    fs.exists(local, function (localExists) {
                        if (localExists) {
                            resolve(local);
                        }
                        else {
                            reject(new Error('Local OmniSharp bin folder not found and OMNISHARP env variable not set'));
                        }
                    });
                });
            }).then(resolve, reject);
        });
    }
});
//# sourceMappingURL=omnisharpLauncher.native.js.map